@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -30,9 +29,10 @@ import fnmatch
 USAGE = """
 \t%prog [OPTIONS] [file1 file2 ... fileN]
 """
-VERSION = '1.10.0'
+VERSION = '1.11.0'
 
 misspellings = {}
+ignore_words = set()
 exclude_lines = set()
 options = None
 file_opener = None
@@ -208,7 +208,8 @@ def parse_options(args):
     parser.set_defaults(colors=sys.stdout.isatty())
     parser.add_option('-d', '--disable-colors',
                       action='store_false', dest='colors',
-                      help='disable colors even when printing to terminal')
+                      help='disable colors even when printing to terminal '
+                      '(always on for Windows)')
     parser.add_option('-c', '--enable-colors',
                       action='store_true', dest='colors',
                       help='enable colors even when not printing to terminal')
@@ -222,6 +223,12 @@ def parse_options(args):
                            'equals "-" then default dictionary "%s" is used. '
                            'This option can be specified multiple times.' %
                       default_dictionary)
+    parser.add_option('-I', '--ignore-words',
+                      action='append', metavar='FILE',
+                      help='File that contains words which will be ignored '
+                           'by codespell. File must contain 1 word per line.'
+                           'Words are case sensitive based on how they are '
+                           'written in codespell_lib/data/dictionary.txt')
     parser.add_option('-r', '--regex',
                       action='store', type='string',
                       help='Regular expression which is used to find words. '
@@ -270,6 +277,10 @@ def parse_options(args):
                            'reliable in detecting encodings other than utf-8, '
                            'iso8859-1 and ascii.')
 
+    parser.add_option('-f', '--check-filenames',
+                      action='store_true', default=False,
+                      help='Check file names as well.')
+
     (o, args) = parser.parse_args(list(args))
 
     if not args:
@@ -284,10 +295,18 @@ def build_exclude_hashes(filename):
             exclude_lines.add(line)
 
 
+def build_ignore_words(filename):
+    with codecs.open(filename, mode='r', buffering=1, encoding='utf-8') as f:
+        for line in f:
+            ignore_words.add(line.strip())
+
+
 def build_dict(filename):
     with codecs.open(filename, mode='r', buffering=1, encoding='utf-8') as f:
         for line in f:
             [key, data] = line.split('->')
+            if key in ignore_words:
+                continue
             data = data.strip()
             fix = data.rfind(',')
 
@@ -386,6 +405,7 @@ def ask_for_word_fix(line, wrongword, misspelling, interactivity):
 
 
 def parse_file(filename, colors, summary):
+    bad_count = 0
     lines = None
     changed = False
     global misspellings
@@ -402,6 +422,40 @@ def parse_file(filename, colors, summary):
         # ignore binary files
         if not os.path.isfile(filename):
             return 0
+        if options.check_filenames:
+            for word in word_regex.findall(filename):
+                lword = word.lower()
+                if lword not in misspellings:
+                    continue
+                fix = misspellings[lword].fix
+                fixword = fix_case(word, misspellings[lword].data)
+
+                if summary and fix:
+                    summary.update(lword)
+
+                cfilename = "%s%s%s" % (colors.FILE, filename, colors.DISABLE)
+                cwrongword = "%s%s%s" % (colors.WWORD, word, colors.DISABLE)
+                crightword = "%s%s%s" % (colors.FWORD, fixword, colors.DISABLE)
+
+                if misspellings[lword].reason:
+                    if quiet_level & QuietLevels.DISABLED_FIXES:
+                        continue
+                    creason = "  | %s%s%s" % (colors.FILE,
+                                              misspellings[lword].reason,
+                                              colors.DISABLE)
+                else:
+                    if quiet_level & QuietLevels.NON_AUTOMATIC_FIXES:
+                        continue
+                    creason = ''
+
+                bad_count += 1
+
+                print("%(FILENAME)s: %(WRONGWORD)s "
+                      " ==> %(RIGHTWORD)s%(REASON)s"
+                      % {'FILENAME': cfilename,
+                         'WRONGWORD': cwrongword,
+                         'RIGHTWORD': crightword, 'REASON': creason})
+
         text = is_text_file(filename)
         if not text:
             if not quiet_level & QuietLevels.BINARY_FILE:
@@ -412,7 +466,6 @@ def parse_file(filename, colors, summary):
         except Exception:
             return 0
 
-    bad_count = 0
     for i, line in enumerate(lines):
         if line in exclude_lines:
             continue
@@ -499,6 +552,11 @@ def parse_file(filename, colors, summary):
     return bad_count
 
 
+def _script_main():
+    """Wrap to main() for setuptools."""
+    return main(*sys.argv[1:])
+
+
 def main(*args):
     """Contains flow control"""
     global options
@@ -520,6 +578,15 @@ def main(*args):
             parser.print_help()
             return 1
 
+    ignore_words_files = options.ignore_words or []
+    for ignore_words_file in ignore_words_files:
+        if not os.path.exists(ignore_words_file):
+            print('ERROR: cannot find ignore-words file: %s' %
+                  ignore_words_file, file=sys.stderr)
+            parser.print_help()
+            return 1
+        build_ignore_words(ignore_words_file)
+
     dictionaries = options.dictionary or [default_dictionary]
     for dictionary in dictionaries:
         if dictionary is "-":
@@ -531,7 +598,7 @@ def main(*args):
             return 1
         build_dict(dictionary)
     colors = TermColors()
-    if not options.colors:
+    if not options.colors or sys.platform == 'win32':
         colors.disable()
 
     if options.summary:
@@ -557,16 +624,20 @@ def main(*args):
 
         if os.path.isdir(filename):
             for root, dirs, files in os.walk(filename):
+                if glob_match.match(root):  # skip (absolute) directories
+                    del dirs[:]
+                    continue
                 for file_ in files:
+                    if glob_match.match(file_):
+                        continue
                     fname = os.path.join(root, file_)
                     if not os.path.isfile(fname) or not os.path.getsize(fname):
                         continue
-                    if glob_match.match(root):  # skips also match directories
-                        continue
-                    if glob_match.match(file_):
-                        continue
                     bad_count += parse_file(fname, colors, summary)
 
+                # skip (relative) directories
+                dirs[:] = [dir_ for dir_ in dirs if not glob_match.match(dir_)]
+
         else:
             bad_count += parse_file(filename, colors, summary)
 
