@@ -10,9 +10,20 @@ import sys
 import tempfile
 import warnings
 
-from nose.tools import assert_equal, assert_true
+from nose.tools import with_setup, assert_equal, assert_true
 
-from codespell_lib import main
+import codespell_lib as cs
+
+
+def reload_codespell_lib():
+    try:
+        reload  # Python 2.7
+    except NameError:
+        try:
+            from importlib import reload  # Python 3.4+
+        except ImportError:
+            from imp import reload  # Python 3.0 - 3.3
+    reload(cs._codespell)
 
 
 def run_codespell(args=(), cwd=None):
@@ -34,164 +45,216 @@ def test_command():
 
 def test_basic():
     """Test some basic functionality"""
-    assert_equal(main('_does_not_exist_'), 0)
+    assert_equal(cs.main('_does_not_exist_'), 0)
     with tempfile.NamedTemporaryFile('w') as f:
-        with CaptureStdout() as sio:
-            assert_equal(main('-D', 'foo', f.name), 1)  # missing dictionary
+        pass
+    with CaptureStdout() as sio:
+        assert_equal(cs.main('-D', 'foo', f.name), 1)  # missing dictionary
+    try:
         assert_true('cannot find dictionary' in sio[1])
-        assert_equal(main(f.name), 0)  # empty file
-        f.write('this is a test file\n')
-        f.flush()
-        assert_equal(main(f.name), 0)  # good
-        f.write('abandonned\n')
-        f.flush()
-        assert_equal(main(f.name), 1)  # bad
-        f.write('abandonned\n')
-        f.flush()
-        assert_equal(main(f.name), 2)  # worse
+        assert_equal(cs.main(f.name), 0)  # empty file
+        with open(f.name, 'a') as f:
+            f.write('this is a test file\n')
+        assert_equal(cs.main(f.name), 0)  # good
+        with open(f.name, 'a') as f:
+            f.write('abandonned\n')
+        assert_equal(cs.main(f.name), 1)  # bad
+        with open(f.name, 'a') as f:
+            f.write('abandonned\n')
+        assert_equal(cs.main(f.name), 2)  # worse
+    finally:
+        os.remove(f.name)
     with TemporaryDirectory() as d:
         with open(op.join(d, 'bad.txt'), 'w') as f:
             f.write('abandonned\nAbandonned\nABANDONNED\nAbAnDoNnEd')
-        assert_equal(main(d), 4)
+        assert_equal(cs.main(d), 4)
         with CaptureStdout() as sio:
-            assert_equal(main('-w', d), 0)
+            assert_equal(cs.main('-w', d), 0)
         assert_true('FIXED:' in sio[1])
         with open(op.join(d, 'bad.txt')) as f:
             new_content = f.read()
-        assert_equal(main(d), 0)
+        assert_equal(cs.main(d), 0)
         assert_equal(new_content, 'abandoned\nAbandoned\nABANDONED\nabandoned')
 
         with open(op.join(d, 'bad.txt'), 'w') as f:
             f.write('abandonned abandonned\n')
-        assert_equal(main(d), 2)
+        assert_equal(cs.main(d), 2)
         with CaptureStdout() as sio:
-            assert_equal(main('-q', '16', '-w', d), 0)
+            assert_equal(cs.main('-q', '16', '-w', d), 0)
         assert_equal(sio[0], '')
-        assert_equal(main(d), 0)
+        assert_equal(cs.main(d), 0)
 
         # empty directory
         os.mkdir(op.join(d, 'test'))
-        assert_equal(main(d), 0)
+        assert_equal(cs.main(d), 0)
 
         # hidden file
         with open(op.join(d, 'test.txt'), 'w') as f:
             f.write('abandonned\n')
-        assert_equal(main(op.join(d, 'test.txt')), 1)
+        assert_equal(cs.main(op.join(d, 'test.txt')), 1)
         os.rename(op.join(d, 'test.txt'), op.join(d, '.test.txt'))
-        assert_equal(main(op.join(d, '.test.txt')), 0)
+        assert_equal(cs.main(op.join(d, '.test.txt')), 0)
 
 
 def test_interactivity():
     """Test interaction"""
+    # Windows can't read a currently-opened file, so here we use
+    # NamedTemporaryFile just to get a good name
     with tempfile.NamedTemporaryFile('w') as f:
-        assert_equal(main(f.name), 0)  # empty file
-        f.write('abandonned\n')
-        f.flush()
-        assert_equal(main('-i', '-1', f.name), 1)  # bad
+        pass
+    try:
+        assert_equal(cs.main(f.name), 0)  # empty file
+        with open(f.name, 'w') as f:
+            f.write('abandonned\n')
+        assert_equal(cs.main('-i', '-1', f.name), 1)  # bad
         with FakeStdin('y\n'):
-            assert_equal(main('-i', '3', f.name), 1)
+            assert_equal(cs.main('-i', '3', f.name), 1)
         with CaptureStdout() as sio:
             with FakeStdin('n\n'):
-                assert_equal(main('-w', '-i', '3', f.name), 0)
+                assert_equal(cs.main('-w', '-i', '3', f.name), 0)
         assert_true('==>' in sio[0])
         with CaptureStdout():
             with FakeStdin('x\ny\n'):
-                assert_equal(main('-w', '-i', '3', f.name), 0)
-        assert_equal(main(f.name), 0)
+                assert_equal(cs.main('-w', '-i', '3', f.name), 0)
+        assert_equal(cs.main(f.name), 0)
+    finally:
+        os.remove(f.name)
+
+    # New example
     with tempfile.NamedTemporaryFile('w') as f:
-        f.write('abandonned\n')
-        f.flush()
-        assert_equal(main(f.name), 1)
+        pass
+    try:
+        with open(f.name, 'w') as f:
+            f.write('abandonned\n')
+        assert_equal(cs.main(f.name), 1)
         with CaptureStdout():
             with FakeStdin(' '):  # blank input -> Y
-                assert_equal(main('-w', '-i', '3', f.name), 0)
-        assert_equal(main(f.name), 0)
+                assert_equal(cs.main('-w', '-i', '3', f.name), 0)
+        assert_equal(cs.main(f.name), 0)
+    finally:
+        os.remove(f.name)
+
     # multiple options
     with tempfile.NamedTemporaryFile('w') as f:
-        f.write('ackward\n')
-        f.flush()
-        assert_equal(main(f.name), 1)
+        pass
+    try:
+        with open(f.name, 'w') as f:
+            f.write('ackward\n')
+
+        assert_equal(cs.main(f.name), 1)
         with CaptureStdout():
             with FakeStdin(' \n'):  # blank input -> nothing
-                assert_equal(main('-w', '-i', '3', f.name), 0)
-        assert_equal(main(f.name), 1)
+                assert_equal(cs.main('-w', '-i', '3', f.name), 0)
+        assert_equal(cs.main(f.name), 1)
         with CaptureStdout():
             with FakeStdin('0\n'):  # blank input -> nothing
-                assert_equal(main('-w', '-i', '3', f.name), 0)
-        assert_equal(main(f.name), 0)
+                assert_equal(cs.main('-w', '-i', '3', f.name), 0)
+        assert_equal(cs.main(f.name), 0)
         with open(f.name, 'r') as f_read:
             assert_equal(f_read.read(), 'awkward\n')
-        f.seek(0)
-        f.write('ackward\n')
-        f.flush()
-        assert_equal(main(f.name), 1)
+        with open(f.name, 'w') as f:
+            f.write('ackward\n')
+        assert_equal(cs.main(f.name), 1)
         with CaptureStdout() as sio:
             with FakeStdin('x\n1\n'):  # blank input -> nothing
-                assert_equal(main('-w', '-i', '3', f.name), 0)
+                assert_equal(cs.main('-w', '-i', '3', f.name), 0)
         assert_true('a valid option' in sio[0])
-        assert_equal(main(f.name), 0)
+        assert_equal(cs.main(f.name), 0)
         with open(f.name, 'r') as f:
             assert_equal(f.read(), 'backward\n')
+    finally:
+        os.remove(f.name)
 
 
 def test_summary():
     """Test summary functionality"""
     with tempfile.NamedTemporaryFile('w') as f:
+        pass
+    try:
         with CaptureStdout() as sio:
-            main(f.name)
+            cs.main(f.name)
         for ii in range(2):
             assert_equal(sio[ii], '')  # no output
         with CaptureStdout() as sio:
-            main(f.name, '--summary')
+            cs.main(f.name, '--summary')
         assert_equal(sio[1], '')  # stderr
         assert_true('SUMMARY' in sio[0])
         assert_equal(len(sio[0].split('\n')), 5)  # no output
-        f.write('abandonned\nabandonned')
-        f.flush()
+        with open(f.name, 'w') as f:
+            f.write('abandonned\nabandonned')
         with CaptureStdout() as sio:
-            main(f.name, '--summary')
+            cs.main(f.name, '--summary')
         assert_equal(sio[1], '')  # stderr
         assert_true('SUMMARY' in sio[0])
         assert_equal(len(sio[0].split('\n')), 7)
         assert_true('abandonned' in sio[0].split()[-2])
+    finally:
+        os.remove(f.name)
+
+
+@with_setup(reload_codespell_lib, reload_codespell_lib)
+def test_ignore_dictionary():
+    """Test ignore dictionary functionality"""
+    with TemporaryDirectory() as d:
+        with open(op.join(d, 'bad.txt'), 'w') as f:
+            f.write('abandonned\nabondon\n')
+        with tempfile.NamedTemporaryFile('w') as f:
+            pass
+        with open(f.name, 'w') as f:
+            f.write('abandonned\n')
+        assert_equal(cs.main('-I', f.name, d), 1)
+
+
+@with_setup(reload_codespell_lib, reload_codespell_lib)
+def test_custom_regex():
+    """Test custom word regex"""
+    with TemporaryDirectory() as d:
+        with open(op.join(d, 'bad.txt'), 'w') as f:
+            f.write('abandonned_abondon\n')
+        assert_equal(cs.main(d), 0)
+        assert_equal(cs.main('-r', "[a-z]+", d), 2)
 
 
 def test_exclude_file():
     """Test exclude file functionality"""
     with TemporaryDirectory() as d:
-        with open(op.join(d, 'bad.txt'), 'w') as f:
-            f.write('abandonned 1\nabandonned 2\n')
+        with open(op.join(d, 'bad.txt'), 'wb') as f:
+            f.write('abandonned 1\nabandonned 2\n'.encode('utf-8'))
+        assert_equal(cs.main(d), 2)
         with tempfile.NamedTemporaryFile('w') as f:
-            f.write('abandonned 1\n')
-            f.flush()
-            assert_equal(main(d), 2)
-            assert_equal(main('-x', f.name, d), 1)
+            pass
+        with open(f.name, 'wb') as f:
+            f.write('abandonned 1\n'.encode('utf-8'))
+        assert_equal(cs.main(d), 2)
+        assert_equal(cs.main('-x', f.name, d), 1)
 
 
 def test_encoding():
     """Test encoding handling"""
     # Some simple Unicode things
     with tempfile.NamedTemporaryFile('wb') as f:
-        # with CaptureStdout() as sio:
-        assert_equal(main(f.name), 0)
-        f.write(u'naïve\n'.encode('utf-8'))
-        f.flush()
-        assert_equal(main(f.name), 0)
-        assert_equal(main('-e', f.name), 0)
-        f.write(u'naieve\n'.encode('utf-8'))
-        f.flush()
-        assert_equal(main(f.name), 1)
-    # Binary file warning
-    with tempfile.NamedTemporaryFile('wb') as f:
-        assert_equal(main(f.name), 0)
-        f.write(b'\x00\x00naiive\x00\x00')
-        f.flush()
+        pass
+    # with CaptureStdout() as sio:
+    assert_equal(cs.main(f.name), 0)
+    try:
+        with open(f.name, 'wb') as f:
+            f.write(u'naïve\n'.encode('utf-8'))
+        assert_equal(cs.main(f.name), 0)
+        assert_equal(cs.main('-e', f.name), 0)
+        with open(f.name, 'ab') as f:
+            f.write(u'naieve\n'.encode('utf-8'))
+        assert_equal(cs.main(f.name), 1)
+        # Binary file warning
+        with open(f.name, 'wb') as f:
+            f.write(b'\x00\x00naiive\x00\x00')
         with CaptureStdout() as sio:
-            assert_equal(main(f.name), 0)
+            assert_equal(cs.main(f.name), 0)
         assert_true('WARNING: Binary file' in sio[1])
         with CaptureStdout() as sio:
-            assert_equal(main('-q', '2', f.name), 0)
+            assert_equal(cs.main('-q', '2', f.name), 0)
         assert_equal(sio[1], '')
+    finally:
+        os.remove(f.name)
 
 
 def test_ignore():
@@ -199,19 +262,28 @@ def test_ignore():
     with TemporaryDirectory() as d:
         with open(op.join(d, 'good.txt'), 'w') as f:
             f.write('this file is okay')
-        assert_equal(main(d), 0)
+        assert_equal(cs.main(d), 0)
         with open(op.join(d, 'bad.txt'), 'w') as f:
             f.write('abandonned')
-        assert_equal(main(d), 1)
-        assert_equal(main('--skip=bad*', d), 0)
-        assert_equal(main('--skip=bad.txt', d), 0)
+        assert_equal(cs.main(d), 1)
+        assert_equal(cs.main('--skip=bad*', d), 0)
+        assert_equal(cs.main('--skip=bad.txt', d), 0)
         subdir = op.join(d, 'ignoredir')
         os.mkdir(subdir)
         with open(op.join(subdir, 'bad.txt'), 'w') as f:
             f.write('abandonned')
-        assert_equal(main(d), 2)
-        assert_equal(main('--skip=bad*', d), 0)
-        assert_equal(main('--skip=*ignoredir*', d), 1)
+        assert_equal(cs.main(d), 2)
+        assert_equal(cs.main('--skip=bad*', d), 0)
+        assert_equal(cs.main('--skip=*ignoredir*', d), 1)
+        assert_equal(cs.main('--skip=ignoredir', d), 1)
+
+
+def test_check_filename():
+    """Test filename check"""
+    with TemporaryDirectory() as d:
+        with open(op.join(d, 'abandonned.txt'), 'w') as f:
+            f.write('.')
+        assert_equal(cs.main('-f', d), 1)
 
 
 class TemporaryDirectory(object):
